/*
 * Copyright 2010-2013 Intel Corporation.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, version 2.1.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * Disclaimer: The codes contained in these modules may be specific
 * to the Intel Software Development Platform codenamed Knights Ferry,
 * and the Intel product codenamed Knights Corner, and are not backward
 * compatible with other Intel products. Additionally, Intel will NOT
 * support the codes or instruction set in future products.
 *
 * Intel offers no warranty of any kind regarding the code. This code is
 * licensed on an "AS IS" basis and Intel is not obligated to provide
 * any support, assistance, installation, training, or other services
 * of any kind. Intel is also not obligated to provide any updates,
 * enhancements or extensions. Intel specifically disclaims any warranty
 * of merchantability, non-infringement, fitness for any particular
 * purpose, and any other warranty.
 *
 * Further, Intel disclaims all liability of any kind, including but
 * not limited to liability for infringement of any proprietary rights,
 * relating to the use of the code, even if Intel is notified of the
 * possibility of such liability. Except as expressly stated in an Intel
 * license agreement provided with this code and agreed upon with Intel,
 * no license, express or implied, by estoppel or otherwise, to any
 * intellectual property rights is granted herein.
 */

// Common code for sink processes that is shared between process sink and daemon

#include <internal/_Process.h>

STATIC_ASSERT( COI_PROCESS_API_VERSION >= COI_PROCESS_API_VERSION_MIN );

COIRESULT _COISinkProcess::SendProcessCreateResponse(_SCIFComm& comm,
                                                    uint64_t process_create_result,
                                                    uint16_t eventPortNum)
{
    // Assume the comm is already connected

    // receive the source's version
    Message_t version_msg;
    COIRESULT result;
    result = comm.ReceiveUnsafe(version_msg);

    assert( version_msg.size() == COI_PROCESS_API_VERSION_STR_MAX_SIZE );

    if( result != COI_SUCCESS )
    {
        return result;
    }

    // sink side must be at least as big as the source
    double source_version_num = atof( version_msg.buffer() );

    COIProcessMessage_t response_msg;
    COIProcessMessage_t::CREATE_RESPONSE_T* create_response;
    response_msg.SetPayload(create_response);

    // Always send back the version on the sink
    strncpy( create_response->sink_version,
            COI_PROCESS_API_VERSION_STR,
            COI_PROCESS_API_VERSION_STR_MAX_SIZE );

    if( source_version_num < COI_PROCESS_API_VERSION_MIN ||
        source_version_num > COI_PROCESS_API_VERSION )
    {
        create_response->version_compatibility_result = COI_VERSION_MISMATCH;
        create_response->process_create_result = COI_ERROR;
        create_response->evtPortNum = (uint16_t)-1;
    }
    else
    {
        create_response->process_create_result = process_create_result;
        create_response->evtPortNum = eventPortNum;
        create_response->version_compatibility_result = COI_SUCCESS;
    }

    return comm.SendUnsafe(response_msg);

}
